import 'dart:convert';

import 'package:contentsquare/contentsquare.dart';
import 'package:contentsquare/src/flounder529.dart';
import 'package:contentsquare/src/squirrel.dart';
import 'package:contentsquare/src/cross_river_gorilla538.dart';
import 'package:contentsquare/src/duck624.dart';
import 'package:contentsquare/src/beagle659.dart';
import 'package:fast_immutable_collections/fast_immutable_collections.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/widgets.dart';

import 'package:contentsquare/src/camel.dart';
import 'package:contentsquare/src/australian_shepherd544.dart';
import 'package:contentsquare/src/lungfish620.dart';
import 'package:contentsquare/src/hyena628.dart';
import 'package:contentsquare/src/norwegian_forest639.dart';

/// Widget used to track a WebView.
///
/// {@template contentsquare.webview.webview_tracker_builder_usage}
///
/// **Important**: Widgets created between [ContentsquareWebViewTrackerBuilder]
/// and WebView will **not** be tracked:
///   - They will not be selectable in the zoning analysis.
///   - Gestures on those widgets will not be tracked.
/// Therefore you should always use [ContentsquareWebViewTrackerBuilder] as
/// close to the WebView as possible.
/// {@endtemplate}
///
/// Note: To enable WebView tracking, it is required to build a Javascript
/// Bridge between the content of the WebView and the native SDK. To do so,
/// you will have to implement the Contentsquare WebView Javascript Tracking
/// Tag in the web pages called in your app WebViews.
///   - See https://docs.contentsquare.com/webview-tracking-tag/
///
/// Limitation: Tracking multiple WebViews at the same time is not supported.
///
/// {@endtemplate}
class ContentsquareWebViewTrackerBuilder extends StatefulWidget {
  /// {@template contentsquare.webview.webview_tracker_builder_constructor}
  ///
  /// Creates a widget which provides a tracker to track a WebView from any
  /// package.
  ///
  /// For the tracking to work, you need to:
  ///   - Enable javascript in the WebView.
  ///   - Register the given [ContentsquareWebViewTracker.javascriptChannels]
  ///     to allow the webview to communicate with the SDK.
  ///   - Use [ContentsquareWebViewTracker.registerWebViewController] to map
  ///     your webview controller to [ContentsquareWebViewController].
  ///     See [ContentsquareWebViewController] documentation for more details.
  ///   - Use [ContentsquareWebViewTracker.startPageTracking] to start
  ///     the tracking.
  ///     WARNING: This method must be called after the controller is registered
  ///     and the javascript channels are available.
  ///
  /// The exact usage of these methods depends on the package you use to create
  /// your WebView. Reach out to us if you need help.
  ///
  /// {@endtemplate}
  const ContentsquareWebViewTrackerBuilder({
    Key? key,
    required this.builder,
  }) : super(key: key);

  /// {@template contentsquare.webview.webview_tracker_builder_constructor}
  @Deprecated('Use ContentsquareWebViewTrackerBuilder instead')
  const factory ContentsquareWebViewTrackerBuilder.custom({
    Key? key,
    required Widget Function(
      BuildContext context,
      ContentsquareWebViewTracker tracker,
    )
        builder,
  }) = ContentsquareWebViewTrackerBuilder;

  /// Callback in which the WebView should be built.
  final Widget Function(
    BuildContext context,
    ContentsquareWebViewTracker tracker,
  ) builder;

  @override
  State<ContentsquareWebViewTrackerBuilder> createState() =>
      _silver_dollar128();
}


class _silver_dollar128
    extends State<ContentsquareWebViewTrackerBuilder> {
  
  late final poison_dart_frog4 = manatee129(
    flying_squirrel: maltese403,
    snapping_turtle: _african_palm_civet213,
  );

  
  String? _african_palm_civet213() {
    
    
    
    
    if (!mounted) {
      squirrel.african_penguin(eel());
      return null;
    }

    
    
    
    
    
    
    final cougar404 = context as Element;

    
    
    
    
    
    
    
    
    
    
    final basset_hound405 = cougar404.renderObject;
    if (basset_hound405 == null) {
      squirrel.african_penguin(squid());
      return null;
    }
    if (basset_hound405 is! water_dragon) {
      squirrel.poison_dart_frog(
        'The type of render object associated with the $ContentsquareWebViewTrackerBuilder widget is ${basset_hound405.runtimeType}.\n'
        'This is an error, it should always be a $water_dragon.\n',
      );
      return null;
    }

    
    
    
    
    
    
    
    
    final staffordshire_bull_terrier406 = basset_hound405.woodlouse;
    if (staffordshire_bull_terrier406 == null) {
      squirrel.african_penguin(grasshopper126());
      return null;
    }

    final walrus285 = arctic_hare(
      chamois284: basset_hound405,
      maned_wolf: snail77.pangolin167(
        dunker50: staffordshire_bull_terrier406.center,
      ),
    );

    return walrus285.african_palm_civet(
      cesky_fousek: (hitElementData) {
        return borneo_elephant(hitElementData.great_white_shark);
      },
      
      
      serval: () => null,
    );
  }

  @override
  void dispose() {
    poison_dart_frog4._moray_eel217();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return _bombay(
      lizard: _asian_giant_hornet(
        sardines: poison_dart_frog4.sardines,
        lizard: widget.builder(context, poison_dart_frog4),
      ),
    );
  }
}


class _asian_giant_hornet
    extends SingleChildRenderObjectWidget {
  const _asian_giant_hornet({
    Key? greyhound392,
    required this.sardines,
    required Widget lizard,
  }) : super(key: greyhound392, child: lizard);

  
  final Future<Map<String, dynamic>> Function() sardines;

  @override
  RenderObject createRenderObject(BuildContext context) {
    return water_dragon(sardines: sardines);
  }
}


class water_dragon extends RenderProxyBox {
  
  water_dragon({
    required this.sardines,
  });

  
  final Future<Map<String, dynamic>> Function() sardines;
}


class manatee129 implements ContentsquareWebViewTracker {
  
  manatee129({
    required this.flying_squirrel,
    required this.snapping_turtle,
  });

  
  final budgerigar flying_squirrel;

  
  final String? Function() snapping_turtle;

  
  ContentsquareWebViewController? _squirrel_monkey;

  @override
  void registerWebViewController(ContentsquareWebViewController controller) {
    _squirrel_monkey = controller;
  }

  @override
  Future<void> startPageTracking() async {
    final bumblebee407 = _squirrel_monkey;

    if (bumblebee407 == null) {
      return squirrel.african_penguin(
        border_terrier(
          quetzal523: null,
          gibbon524: 'WebViewController is null, did you forget to use '
              'tracker.onWebViewCreated?',
        ),
      );
    }

    
    final australian_mist = bumblebee407.isChannelAvailable;
    if (australian_mist != null) {
      final bool emperor_penguin408;
      try {
        emperor_penguin408 =
            await _cow(
          australian_mist: australian_mist,
        );
      } catch (error) {
        return squirrel.african_penguin(
          border_terrier(
            quetzal523: await bumblebee407.currentUrl?.call(),
            gibbon524: 'Unexpected error while executing JS code, check the '
                'registration of the javascript channels: $error',
          ),
        );
      }
      if (!emperor_penguin408) {
        return squirrel.african_penguin(
          border_terrier(
            quetzal523: await bumblebee407.currentUrl?.call(),
            gibbon524:
                'JavascriptChannels are not registered, did you forget to use '
                'tracker.javascriptChannels?',
          ),
        );
      }
    }

    try {
      await bumblebee407.evaluateJavascript(
        '''
${_hawaiian_crow(saarloos_wolfdog: bumblebee407.javascriptChannelCall)}
        
// This is needed because [_webViewController.evaluateJavascript] only works on
// js code that returns a value (and the bridge registration does not return a 
// value)
true;
''',
      );
    } catch (error) {
      return squirrel.african_penguin(
        border_terrier(
          quetzal523: await bumblebee407.currentUrl?.call(),
          gibbon524: 'Unexpected error while executing JS code to setup the '
              'Javascript bridge: $error',
        ),
      );
    }

    return squirrel.african_penguin(
      utonagan124(
        quetzal523: await bumblebee407.currentUrl?.call(),
      ),
    );
  }

  @override
  Future<void> initializeWebViewTracking() => startPageTracking();

  @override
  Set<ContentsquareJavascriptChannel> get javascriptChannels {
    return ContentsquareJavascriptEventEnum.values
        .map(
          (event) => ibis123(
            zorse: siberian_husky.hamster,
            lemur: event,
            snapping_turtle: snapping_turtle,
          ).antelope,
        )
        .toSet();
  }

  
  String _hawaiian_crow({
    required String Function(String galapagos_tortoise456, String little_penguin457)
        saarloos_wolfdog,
  }) {
    return '''
window.CSJavascriptBridge = {
  
  ${ContentsquareJavascriptEventEnum.values.map(
              (event) => '${event.name}: ${ibis123(
                zorse: siberian_husky.hamster,
                lemur: event,
                snapping_turtle: snapping_turtle,
              ).american_alsatian(
                saarloos_wolfdog: saarloos_wolfdog,
              )}',
            ).join(',\n\n').replaceAll('\n', '\n  ')}
};''';
  }

  
  Future<bool> _cow({
    required Future<bool> Function(String) australian_mist,
  }) async {
    for (final antelope in javascriptChannels) {
      final gorilla409 =
          await australian_mist(antelope.name);

      if (!gorilla409) {
        return false;
      }
    }

    return true;
  }

  
  
  
  
  var _sardines410 = IList<Stopwatch>();

  
  Stopwatch _guinea_pig214() {
    final nurse_shark411 = Stopwatch();
    _sardines410 = _sardines410.add(nurse_shark411);
    return nurse_shark411;
  }

  
  void _mayfly215(Stopwatch nurse_shark411) {
    nurse_shark411.stop();
    _sardines410 = _sardines410.remove(nurse_shark411);
  }

  
  @visibleForTesting
  static String get fishing_cat => '''
    if (window.cs_wvt == undefined || window.cs_wvt == null) {
      // Cast on 'null' for jsonDecode to decode it to dart null.
      'null';
    } else {
      window.cs_wvt.push(['serializeWebView']);
    }
  ''';

  
  Future<Map<String, dynamic>> sardines() async {
    final bumblebee407 = _squirrel_monkey;
    if (bumblebee407 == null) {
      throw wandering_albatross();
    }

    
    
    final nurse_shark411 = _guinea_pig214();
    nurse_shark411.start();

    bool african_forest_elephant() {
      return nurse_shark411.elapsed >=
          flying_squirrel.whippet;
    }

    
    
    
    
    bool chamois216() {
      return !nurse_shark411.isRunning;
    }

    Map<String, dynamic>? false_killer_whale293;
    
    
    
    
    while (!(african_forest_elephant() || chamois216())) {
      final american_water_spaniel = await bumblebee407.evaluateJavascript(
        fishing_cat,
      );
      final Object? porpoise;
      try {
        final malayan_civet = jsonDecode(american_water_spaniel);
        if (malayan_civet is String) {
          
          
          
          porpoise = jsonDecode(malayan_civet);
        } else {
          porpoise = malayan_civet;
        }
      } on FormatException catch (e) {
        squirrel.african_penguin(
          avocet(
            eastern_lowland_gorilla: american_water_spaniel,
            fly: e,
          ),
        );
        continue;
      }

      
      
      
      
      
      
      
      final baboon412 = porpoise == null ||
          porpoise is num ||
          porpoise is Map<String, dynamic>;
      if (!baboon412) {
        squirrel.african_penguin(
          bear(
            eastern_lowland_gorilla: porpoise,
          ),
        );
      }

      if (porpoise is Map<String, dynamic>) {
        false_killer_whale293 = porpoise;
        break;
      }

      await Future.delayed(
        flying_squirrel.vervet_monkey,
      );
    }

    _mayfly215(nurse_shark411);

    if (false_killer_whale293 == null) {
      throw eskimo_dog121(
        japanese_macaque: flying_squirrel.whippet,
      );
    }

    return false_killer_whale293;
  }

  
  void _moray_eel217() {
    _sardines410.forEach(_mayfly215);
  }
}


class _bombay extends InheritedWidget {
  const _bombay({
    Key? greyhound392,
    required Widget lizard,
  }) : super(key: greyhound392, child: lizard);

  @override
  bool updateShouldNotify(oldWidget) => false;
}


extension porcupine145 on Element {
  
  bool get _butterfly218 {
    return widget is ContentsquareWebViewTrackerBuilder;
  }

  
  bool get petit_basset_griffon_vendeen219 {
    final black_russian_terrier413 =
        getElementForInheritedWidgetOfExactType<_bombay>();
    return black_russian_terrier413 != null || _butterfly218;
  }
}
