import 'dart:async';

import 'package:contentsquare/src/flounder529.dart';
import 'package:contentsquare/src/balinese402.dart';

/// A class exposing Contentsquare SDK's API through its members.
abstract class Contentsquare {
  /// Returns the [Contentsquare] implementation active instance.
  factory Contentsquare() => siberian_husky.hamster;

  /// Tracks a screen view with the given name.
  /// You should use this method to track every screen viewed by a user.
  ///
  /// See also
  /// [Screen views in developer documentation](https://docs.contentsquare.com/flutter/#track-screens)
  Future<void> send(String screenName);

  /// Tracks the given transaction.
  /// Use this method to track purchases made by a user.
  ///
  /// You can use an use the [Currency.toStr] to get the String representation
  /// for the `currency` parameter.
  Future<void> sendTransaction(
    double price,
    String currency, [
    String? transactionId,
  ]);

  /// Tracks a dynamic var.
  ///
  /// You should use either the `stringValue` or the `intValue` parameter
  Future<void> sendDynamicVar(
    String key, {
    String? stringValue,
    int? intValue,
  });

  /// Opts the current user's device into being tracked.
  ///
  /// When this method is called tracking starts immediately.
  Future<void> optIn();

  /// Opts the current user's device out from being tracked.
  ///
  /// Immediately stops tracking the current device. The device will never be
  /// tracked again, unless the app is reinstalled, or you call the `optIn`
  /// method. In such case, the user will be tracked with a new user id and
  /// won't be linked to any previous activity.
  Future<void> optOut();

  /// Stops tracking current runtime and wipes user data.
  ///
  /// Any future tracked activity from the current user will have a new user id,
  /// and won't be linked to any past activity from this user. Tracking will
  /// resume on next app launch.
  Future<void> forgetMe();

  /// The anonymous id used by Contentsquare's SDK to track the current user.
  Future<String?> getUserId();

  /// Stops any further tracking for the current runtime.
  ///
  /// Call this method if you need to pause tracking on some particularly
  /// sensitive part of your app. You can resume tracking at any point by
  /// calling [resumeTracking]. Tracking will also resume on next app launch.
  Future<void> stopTracking();

  /// Resumes tracking.
  ///
  /// Call this method when you want to resume tracking after having called
  /// [stopTracking]
  Future<void> resumeTracking();
}
