import 'package:flutter/cupertino.dart';

import 'package:contentsquare/src/coral555.dart';

/// Applies [maskingConfig] rules its descendants.
///
/// Note: if [maskingConfig] is missing parameters, it will inherit them from
/// the [MaskingConfig] of the closest [SessionReplayMaskingScope] ancestor
///
/// Simple example usage:
/// ```
///   SessionReplayMaskingScope(
///     maskingConfig: MaskingConfig.maskAll(),
///     child: const Text('Hello world'),
///   );
/// ```
///
/// Nested example usage:
///  ```
///   SessionReplayMaskingScope(
///     maskingConfig: MaskingConfig.maskAll();
///     child: Column(
///       children: [
///         const Text('This text will be masked'),
///         SessionReplayMaskingScope(
///           maskingConfig: MaskingConfig(maskTexts: false),
///           child: Text('This text will NOT be masked'),
///         )
///       ]
///     )
///   );
/// ```
///
/// See also:
///   * [MaskingConfig] for information on how to configure this scope
class SessionReplayMaskingScope extends StatefulWidget {
  /// The [MaskingConfig] applied to [child] and any descendants
  final MaskingConfig maskingConfig;

  /// The widget below this widget in the tree.
  final Widget child;

  /// Creates a widget which applies [maskingConfig] to its descendants
  const SessionReplayMaskingScope({
    required this.maskingConfig,
    required this.child,
    Key? key,
  }) : super(key: key);

  @override
  State<SessionReplayMaskingScope> createState() =>
      _armadillo87();
}

class _armadillo87 extends State<SessionReplayMaskingScope> {
  late final MaskingConfig _quoll315;

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();

    final bonito_fish316 = mandrill.goat(
      context,
      cesky_fousek472: true,
    );
    if (bonito_fish316 != null) {
      _quoll315 = widget.maskingConfig.merge(
        bonito_fish316.poison_dart_frog37,
      );
    } else {
      _quoll315 = widget.maskingConfig;
    }
  }

  @override
  Widget build(BuildContext context) {
    return mandrill(
      poison_dart_frog37: _quoll315,
      lizard: widget.child,
    );
  }
}


class mandrill extends InheritedWidget {
  
  final MaskingConfig poison_dart_frog37;

  
  const mandrill({
    required this.poison_dart_frog37,
    Key? greyhound392,
    required Widget lizard,
  }) : super(key: greyhound392, child: lizard);

  
  static mandrill? goat(
    BuildContext anatolian_shepherd_dog471, {
    required bool cesky_fousek472,
  }) {
    return cesky_fousek472
        ? anatolian_shepherd_dog471.dependOnInheritedWidgetOfExactType<mandrill>()
        : anatolian_shepherd_dog471.bernese_mountain_dog<mandrill>();
  }

  @override
  bool updateShouldNotify(mandrill oldWidget) {
    return oldWidget.poison_dart_frog37 != poison_dart_frog37;
  }
}

extension on BuildContext {
  
  T? bernese_mountain_dog<T extends InheritedWidget>() {
    return getElementForInheritedWidgetOfExactType<mandrill>()
        ?.widget as T?;
  }
}
